package counter;

/**
 * Illustrates use of semaphores
 * 
 * Lecture: Condition Objects
 * 
 * $Id: BoundedCounterSem.java 24407 2009-01-26 20:22:31Z oscar $
 *
 */
public class BoundedCounterSem extends BoundedCounterAbstract {
	protected Semaphore mutex;
	protected Semaphore full;
	protected Semaphore empty;
	
	BoundedCounterSem() {
		mutex = new Semaphore(1);
		full = new Semaphore(0); // number of items
		empty = new Semaphore(MAX-MIN); // number of slots
	}
	
	public long value() {
		mutex.down();	// grab the resource
		long val = count;
		mutex.up();		// release it
		return val;
	}
	
	public void inc() {
		empty.down();	// grab a slot
		mutex.down();
		count ++;
		mutex.up();
		full.up();		// release an item
		checkInvariant();
	}
	
	public void dec() {
		full.down();	// grab an item
		mutex.down();
		count --;
		mutex.up();
		empty.up();		// release a slot
		checkInvariant();
	}
	
	/*
	 * These would cause a nested monitor problem!
	 */
	public void BADinc() {
		mutex.down();
		empty.down();	// grab a slot
		count ++;
		full.up();		// release an item
		mutex.up();
		checkInvariant();
	}
	
	public void BADdec() {
		mutex.down();
		full.down();	// grab an item
		count --;
		empty.up();		// release a slot
		mutex.up();
		checkInvariant();
	}

}
